"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _JSONParseStream_promises, _JSONParseStream_queues;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ND_JSONParseStream = exports.JSONParseStream = void 0;
// deno-lint-ignore-file no-explicit-any
const resolvable_promise_1 = require("@worker-tools/resolvable-promise");
const whatwg_stream_to_async_iter_1 = require("whatwg-stream-to-async-iter");
const json_parser_js_1 = require("./json-parser.js");
const json_path_js_1 = require("./json-path.js");
const async_queue_js_1 = require("./async-queue.js");
const split_stream_js_1 = require("./split-stream.js");
async function* identity(iter) {
    for await (const x of iter)
        yield x;
}
/**
 *
 */
class JSONParseStream extends TransformStream {
    constructor(jsonPath = '$.*') {
        let parser;
        const expr = (0, json_path_js_1.normalize)(jsonPath);
        super({
            start: (controller) => {
                parser = new json_parser_js_1.JSONParser();
                parser.onValue = (value) => {
                    const path = [...parser.stack.map(_ => _.key), parser.key]; // TODO: modify parser to provide key efficiently
                    path[0] || (path[0] = '$');
                    const nPath = (0, json_path_js_1.normalize)(path.join('.')); // FIXME: avoid string concatenation/joining
                    if ((0, json_path_js_1.match)(expr, nPath)) {
                        controller.enqueue(value);
                    }
                    // FIXME: use trie for better performance!?
                    for (const expr of __classPrivateFieldGet(this, _JSONParseStream_promises, "f").keys()) {
                        if ((0, json_path_js_1.match)(expr, nPath)) {
                            __classPrivateFieldGet(this, _JSONParseStream_promises, "f").get(expr).resolve(value);
                            __classPrivateFieldGet(this, _JSONParseStream_promises, "f").delete(expr);
                        }
                    }
                    for (const expr of __classPrivateFieldGet(this, _JSONParseStream_queues, "f").keys()) {
                        if ((0, json_path_js_1.match)(expr, nPath)) {
                            __classPrivateFieldGet(this, _JSONParseStream_queues, "f").get(expr).push(value);
                        }
                    }
                };
            },
            transform: (chunk) => {
                parser.write(chunk);
            },
            flush: async () => {
                await Promise.all([
                    ...[...__classPrivateFieldGet(this, _JSONParseStream_promises, "f").values()].map(q => q.resolve(undefined)),
                    ...[...__classPrivateFieldGet(this, _JSONParseStream_queues, "f").values()].map(q => q.return()),
                ]);
            },
        });
        _JSONParseStream_promises.set(this, new Map());
        _JSONParseStream_queues.set(this, new Map());
    }
    promise(jsonPath) {
        if (this.readable.locked)
            throw Error('Already locked');
        const p = new resolvable_promise_1.ResolvablePromise();
        __classPrivateFieldGet(this, _JSONParseStream_promises, "f").set((0, json_path_js_1.normalize)(jsonPath), p);
        return Promise.resolve(p);
    }
    iterable(jsonPath) {
        if (this.readable.locked)
            throw Error('Already locked');
        const q = new async_queue_js_1.AsyncQueue();
        __classPrivateFieldGet(this, _JSONParseStream_queues, "f").set((0, json_path_js_1.normalize)(jsonPath), q);
        return identity(q);
    }
    stream(jsonPath) {
        return (0, whatwg_stream_to_async_iter_1.asyncIterToStream)(this.iterable(jsonPath));
    }
}
exports.JSONParseStream = JSONParseStream;
_JSONParseStream_promises = new WeakMap(), _JSONParseStream_queues = new WeakMap();
/** @deprecated Untested */
class ND_JSONParseStream extends TransformStream {
    constructor() {
        let splitStream;
        let writer;
        let decoder;
        super({
            start(controller) {
                splitStream = new split_stream_js_1.BinarySplitStream();
                writer = splitStream.writable.getWriter();
                decoder = new TextDecoder();
                (async () => {
                    try {
                        for await (const line of (0, whatwg_stream_to_async_iter_1.streamToAsyncIter)(splitStream.readable)) {
                            const sLine = decoder.decode(line).trim();
                            if (sLine)
                                controller.enqueue(JSON.parse(sLine));
                        }
                    }
                    catch (err) {
                        writer.abort(err);
                    }
                })();
            },
            transform(chunk) {
                writer.write(chunk);
            },
            flush() {
                writer.close();
            },
        });
    }
}
exports.ND_JSONParseStream = ND_JSONParseStream;
//# sourceMappingURL=json-parse-stream.js.map