"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsonStringifyStream = exports.jsonStringifyGenerator = void 0;
// deno-lint-ignore-file no-explicit-any
const whatwg_stream_to_async_iter_1 = require("whatwg-stream-to-async-iter");
const _isIterable = (x) => x != null && typeof x === 'object' && Symbol.iterator in x;
const isAsyncIterable = (x) => x != null && typeof x === 'object' && Symbol.asyncIterator in x;
const isPromiseLike = (x) => x != null && typeof x === 'object' && 'then' in x && typeof x.then === 'function';
const isToJSON = (x) => x != null && typeof x === 'object' && 'toJSON' in x;
const safeAdd = (seen, value) => {
    if (seen.has(value))
        throw TypeError('Converting circular structure to JSON');
    seen.add(value);
};
// TODO: Add replacer
// TODO: add formatting/spaces
async function* jsonStringifyGenerator(value, seen = new WeakSet()) {
    if (isAsyncIterable(value)) {
        yield '[';
        safeAdd(seen, value);
        let first = true;
        for await (const v of value) {
            if (!first)
                yield ',';
            else
                first = false;
            yield* jsonStringifyGenerator(v, seen);
        }
        seen.delete(value);
        yield ']';
    }
    else if (isPromiseLike(value)) {
        const v = await value;
        if (v !== undefined) {
            safeAdd(seen, value);
            yield* jsonStringifyGenerator(v, seen);
            seen.delete(value);
        }
    }
    else if (isToJSON(value)) {
        const v = JSON.stringify(value);
        if (v !== undefined)
            yield v;
    }
    else if (Array.isArray(value)) {
        yield '[';
        safeAdd(seen, value);
        let first = true;
        for (const v of value) {
            if (!first)
                yield ',';
            else
                first = false;
            yield* jsonStringifyGenerator(v, seen);
        }
        seen.delete(value);
        yield ']';
    }
    else if (value != null && typeof value === 'object') {
        yield '{';
        safeAdd(seen, value);
        let first = true;
        for (const [k, v] of Object.entries(value)) {
            if (v !== undefined) {
                const generator = jsonStringifyGenerator(v, seen);
                const peek = await generator.next();
                if (peek.value !== undefined) {
                    if (!first)
                        yield ',';
                    else
                        first = false;
                    yield `${JSON.stringify(k)}:`;
                    yield peek.value;
                    yield* generator;
                }
            }
        }
        seen.delete(value);
        yield '}';
    }
    else {
        yield value === undefined ? 'null' : JSON.stringify(value);
    }
}
exports.jsonStringifyGenerator = jsonStringifyGenerator;
function jsonStringifyStream(value) {
    return (0, whatwg_stream_to_async_iter_1.asyncIterableToStream)(jsonStringifyGenerator(value));
}
exports.jsonStringifyStream = jsonStringifyStream;
//# sourceMappingURL=json-stringify.js.map